"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultGeneratorMapper = exports.generatorChain = void 0;
const tslib_1 = require("tslib");
const generator_1 = require("../actions/generator");
/**
 * Chatting with the plop maintainers over on https://github.com/plopjs/plop/issues/214 about best practices when it
 * comes to programmatically running a sequence of steps, chaining answers, args etc. Basically it's not supported.
 *
 * My goal with generatorChain is to run generators in order, ie. running Generator 1's prompts and actions, followed by
 * Generator 2's prompts and actions, passing the data from the previous run onto the next. Enabling this allows us to
 * create more modular Generators, with specific remits, create vs configure workspace as an example. A user may create
 * and configure a brand new workspace in one flow, however if they have an existing workspace, they may want to simply
 * re-configure it. So we should be able to launch the configure generator only with the pre-data points it needs, or
 * jump to any point in a chain and run them all sequentially from there.
 *
 * This generatorChain needs to return what appears to be a normal generator interface to plop so it thinks it's running
 * a single generator.
 */
const generatorChain = (plop, description, generators, bypass, actions = [' Done.']) => {
    return {
        description,
        /**
         * The answers from one iteration are used as previous answers input to the next, so we need to turn this rule off.
         */
        /* eslint-disable no-await-in-loop */
        prompts(inquirer, prevAns = {}) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                let allAns = Object.assign({}, prevAns);
                for (const generator of generators) {
                    const [data, outcome] = yield generator_1.GeneratorActions.run(generator, allAns, plop, bypass);
                    allAns = Object.assign(Object.assign({}, allAns), data);
                }
                return allAns;
            });
        },
        actions,
    };
};
exports.generatorChain = generatorChain;
/*
By default we return generator as is
*/
const defaultGeneratorMapper = (generator) => generator;
exports.defaultGeneratorMapper = defaultGeneratorMapper;
